@@ -3,7 +3,6 @@ import functools
 import itertools
 
 import torch
-from ..backends.thnn import backend as thnn_backend
 from ..parameter import Parameter
 import torch.utils.hooks as hooks
 
@@ -69,17 +68,12 @@ class Module(object):
     _version = 1
 
     def __init__(self):
-        self._construct()
-        # initialize self.training separately from the rest of the internal
-        # state, as it is managed differently by nn.Module and ScriptModule
-        self.training = True
-
-    def _construct(self):
         """
         Initializes internal Module state, shared by both nn.Module and ScriptModule.
         """
         torch._C._log_api_usage_once("python.nn_module")
-        self._backend = thnn_backend
+
+        self.training = True
         self._parameters = OrderedDict()
         self._buffers = OrderedDict()
         self._backward_hooks = OrderedDict()
